class Future extends Modal 
{
	public int setStartTime(int st)
	{
		return UserInterface.getMinTime();
	}

	public int setEndTime(int et)
	{
		int min_time = UserInterface.getMinTime();

		et=et-1;

		if (et<min_time)
			et=min_time;
		
		return et;
	}

	/* we keep all the rows */
	public boolean includeRow(int st, int et)
	{
		int min_time = UserInterface.getMinTime();

		if (st==min_time && et==min_time)
		{
			return false;
		}
		else
			return true;
		
	}
	
} 
