import java.io.*;

class TestTree 
{
	public static void main(String[] args) 
	{
		TreeNode tree;
		Table output = new Table();

		//for (int i=0; i<args.length; i++)
		//{
		//	System.out.print(args[i]+" | ");
		//}
		//System.out.println();

		tree=Tree.buildTree(args, false);
		PrintTree.print(tree);
		System.out.println();
		output=ExecuteTree.generate(tree);
		PrintTable.print(output);
		File fo = new File("results.xml");
		SaveXMLfile.save(output, fo);
		//PrintRes.print(output);
		System.out.println();
	}
}
