class ProjNode extends UnaryNode 
{
	String[] arg;

	public ProjNode(String[] a, TreeNode t)
	{
		super(t);
		arg=a;
	}

	public String[] getArg()
	{
		return this.arg;
	}

	public void setArg(String[] a)
	{
		this.arg=a;
	}

	public String getField(int pos)
	{
		return this.arg[pos];
	}

	public int getArgLen()
	{
		return this.arg.length;
	}

	public TreeNode getNode()
	{
		return this;
	}
}
