class OnWithNode extends UnaryNode 
{
	String[] tabACol;
	String[] tabBCol;

	public OnWithNode(String[] a, String[] b, TreeNode t)
	{
		super(t);
		tabACol=a;
		tabBCol=b;
	}

	public String[] getTabACol()
	{
		return this.tabACol;
	}

	public String[] getTabBCol()
	{
		return this.tabBCol;
	}

	public TreeNode getNode()
	{
		return this;
	}
}
