class CondNode extends UnaryNode 
{
	String arg;
	String op;
	String val;

	public CondNode(String a, String o, String v, TreeNode t)
	{
		super(t);
		arg=a;
		op=o;
		val=v;
	}

	public String getArg()
	{
		return this.arg;
	}

	public void setArg(String a)
	{
		this.arg=a;
	}

	public String getOp()
	{
		return this.op;
	}

	public void setOp(String o)
	{
		this.op=o;
	}

	public String getVal()
	{
		return this.val;
	}

	public void setVal(String v)
	{
		this.val=v;
	}

	public TreeNode getNode()
	{
		return this;
	}
}
