class AtTimeNode extends UnaryNode 
{
	String arg;

	public AtTimeNode(String a, TreeNode t)
	{
		super(t);
		arg=a;
	}

	public String getArg()
	{
		return this.arg;
	}

	public void setArg(String a)
	{
		this.arg=a;
	}

	public TreeNode getNode()
	{
		return this;
	}
}
