class UnaryNode extends TreeNode
{
	TreeNode next;

	public UnaryNode(TreeNode t)
	{
		next=t;
	}

	public TreeNode getNext()
	{
		return this.next;
	}

	public void setNext(TreeNode t)
	{
		this.next=t;
	}

	public TreeNode getNode()
	{
		return this;
	}

	public String getType()
	{
		return "UnaryNode";
	}

}