class AlwaysPast extends Previous 
{
	/* uses the same method body for 'setStartTime(st)' as defined by the class 'Previous' */
	
	/* uses the same method body for 'setEndTime(et)' as defined by the class 'Previous' */

	/* we keep all the rows/evaluate rows where the start time of the
	 * row is the minimum time known to the temporal database system
	 */
	public boolean includeRow(int st, int et)
	{
		int min_time = UserInterface.getMinTime();

		if (st==min_time)
			return true;
		else 
			return false;
	}
	
	
}
