import java.util.*;
import java.lang.*;

class QryBuffer {
  ArrayList qryBuffer = new ArrayList();
  int index=0;
  int counter=0;

  public String  getQueryTitle() {
    return "Query "+index;
  }

  public String getCounter() {
    Integer count = new Integer(counter);
    String s = count.toString();
    return s;
  }

  public void setQry(String s) {
    qryBuffer.add(s);
    counter++;
    index=counter-1;
  }

  public String getQry() {
    return getQry(index);
  }

  public String getQry(int pos) {
    if (pos<getSize() && pos>=0)
      return (String)qryBuffer.get(pos);
    else
      return "";
  }

  public int getSize() {
    return qryBuffer.size();
  }

  public String prevQry() {
    if (--index<0)
      index=0;
    
    return getQry(index);
  }

  public String nextQry() {
    if (++index>getSize())
      index=getSize();

    return getQry(index);
  }

  public void print() {
    for (int i=0; i<getSize(); i++) {
      String s=getQry(i);
      System.out.println(i+"="+s);
    }
  }
}
