class TimeDefinition 
{
	private static int minTime = 0;
	private static int maxTime = 9;

	public static int getMinTime()
	{
		return minTime;
	}

	public static int getMaxTime()
	{
		return maxTime;
	}

	public static void setMinTime(int t)
	{
		minTime = t;
	}

	public static void setMaxTime(int t)
	{
		maxTime = t;
	}
}
