class IndexTest
{
	public static void main(String[] args) 
	{
		Table R=new Table();
		R=Query.execQuery("Select * from movement");
		String[] name = {"no"}; //R.col.getColNames()
		int[] pos = AuxFuncs.orderCols(name, R.getCol());
		PrintTable.print(R);

		AuxFuncs.quicksort(R, 0, R.getNoRowSet()-1, pos);
		PrintTable.print(R);

		System.out.print("[");
		for (int i=0; i<pos.length; i++)
		{
			System.out.print(" "+ pos[i]);
		}
		System.out.println(" ]");
	}
}
