class AlwaysFuture extends Next
{

	/* uses the same method body for 'setStartTime(st)' as defined by the class 'Next' */
	
	/* uses the same method body for 'setEndTime(et)' as defined by the class 'Next' */

	/* we keep all the rows/evaluate rows where the end time of the
	 * row is the maximum time known to the temporal database system
	 */
	public boolean includeRow(int st, int et)
	{
		int max_time = UserInterface.getMaxTime();

		if (et==max_time)
			return true;
		else 
			return false;
	}	

}
