class RenameNode extends UnaryNode 
{
	String[] oldCol;
	String[] newCol;

	public RenameNode(String[] o, String[] n, TreeNode t)
	{
		super(t);
		oldCol=o;
		newCol=n;
	}

	public String[] getOldCol()
	{
		return this.oldCol;
	}

	public String[] getNewCol()
	{
		return this.newCol;
	}

	public TreeNode getNode()
	{
		return this;
	}
}
