class ModalNode extends UnaryNode 
{
	String operator;

	public ModalNode(String op, TreeNode t)
	{
		super(t);
		operator=op;
	}

	public String getOp()
	{
		return this.operator;
	}

	public void setOp(String op)
	{
		this.operator=op;
	}

	public TreeNode getNode()
	{
		return this;
	}
}
