class Past extends Modal 
{
	public int setStartTime(int st)
	{
		int max_time = UserInterface.getMaxTime();

		st=st+1;

		if (st>max_time)
			st=max_time;
		
		return st;
	}

	public int setEndTime(int et)
	{
		return UserInterface.getMaxTime();
	}
	
	/* we keep all the rows */
	public boolean includeRow(int st, int et)
	{
		int max_time = UserInterface.getMaxTime();

		if (st==max_time && et==max_time)
		{
			return false;
		}
		else
			return true;
	}
}
