class BinaryNode extends TreeNode
{
	String operator ;
	TreeNode leftTree;
	TreeNode rightTree;

	public BinaryNode(String op, TreeNode lt, TreeNode rt)
	{
		operator=op;
		leftTree=lt;
		rightTree=rt;
	}

	public String getOp()
	{
		return this.operator;
	}

	public void setOp(String op)
	{
		this.operator=op;
	}

	public TreeNode getLeftTree()
	{
		return this.leftTree;
	}

	public TreeNode getRightTree()
	{
		return this.rightTree;
	}

	public void setLeftTree(TreeNode lt)
	{
		this.leftTree=lt;
	}

	public void setRightTree(TreeNode rt)
	{
		this.rightTree=rt;
	}

	public TreeNode getNode()
	{
		return this;
	}

	public String getType()
	{
		return "BinaryNode";
	}
}
