class Next extends Future
{
	public int setStartTime(int st)
	{
		int min_time = UserInterface.getMinTime();

		st=st-1;

		if (st<min_time)
			st=min_time;
		
		return st;
	}

	/* uses the same method body for 'setEndTime(et)' as defined by the class 'Future' */
	
}
