class TestComp 
{
	public static void main(String[] args) 
	{
		
		Table R=new Table();
		R=Query.execQuery("Select * from account");
		String[] oldName = {"types"};
		String[] newName = {"jay"};

		R=RenameField.change(R, oldName, newName);
		PrintTable.print(R);

		System.out.println("uppercase:" +oldName[0].toUpperCase());
		System.out.println("lowercase:" +oldName[0]);
	}
}
