class CatchErr
{
	// Bracket and nested query handling
	public static void brackErr(int noBracks) 
	{
		System.out.print("Handling of sub expression: ");
		if (noBracks>0)
			System.out.println("Missing bracket ')'");
		else
			System.out.println("Missing bracket '('");
	}

	
	// Invalid token handling
	public static void tokenErr(String tok, String function) 
	{
		System.out.println("Invalid use of token='"+tok+"' in method "+function);
	}

	// Missing a token or an argument value
	public static void missErr(String type, String function) 
	{
		System.out.println("Missing '"+type+"' value for executing operator "+function);
	}


	public static void formatErr(String function) 
	{
			System.out.println("ERROR in performing "+function+": The two tables have different column formats");
			System.out.println("ASSUMPTION MADE: The "+function+" operation has been neglected");
			System.out.println("	The expression on the LEFT of the "+function+" has been evaluated");
			System.out.println("	The expression on the RIGHT of the "+function+" has been ignored");
	}

	public static void columnErr(String column, String function) 
	{
			System.out.println("ERROR in performing "+function+": Cannot locate column '"+column+"'");
			System.out.println("ASSUMPTION MADE: The column '"+column+"' has been ignored for the executing operator "+function);
	}

	public static void joinNoComColsErr(String function) 
	{
			System.out.println("ERROR in performing "+function+": The two tables have no columns in common");
			System.out.println("ASSUMPTION MADE: The "+function+" operation has been neglected");
			System.out.println("	The expression on the LEFT of the "+function+" has been evaluated");
			System.out.println("	The expression on the RIGHT of the "+function+" has been ignored");
			//System.out.println("ASSUMPTION MADE: A PRODUCT of the corresponding JOIN type is performed");
	}
	
}
