import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class UserFrame
{
	public static void main(String[] args) 
	{
		//UserInterface usrframe = new UserInterface();
		
		//WindowQuitter wquit = new WindowQuitter(); /* construct a listener for the frame */
		//usrframe.addWindowListener(wquit); /* register the listener */

		//usrframe.setSize(400, 375);
		//usrframe.setVisible(true);	
		JFrame.setDefaultLookAndFeelDecorated(true);

		JFrame frame = new JFrame("Temporal Database System");

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

	UserInterface applet = new UserInterface();
	frame.setContentPane(applet);
        frame.pack();
        frame.setVisible(true);

	}

}

