class Previous extends Past
{
	/* uses the same method body for 'setStartTime(st)' as defined by the class 'Past' */

	public int setEndTime(int et)
	{
		int max_time = UserInterface.getMaxTime();

		et=et+1;

		if (et>max_time)
			et=max_time;
		
		return et;
	}
	
}
